@@ -13,10 +13,9 @@ You should have received a copy of the GNU General Public License along
 with NML; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA."""
 
-from nml import generic
+from nml import generic, global_constants
 from nml.actions import base_action
 from nml.ast import base_statement, general
-import nml
 
 total_action2_ids = 0x100
 free_action2_ids = list(range(0, total_action2_ids))
@@ -24,7 +23,8 @@ free_action2_ids = list(range(0, total_action2_ids))
 """
 Statistics about spritegroups.
 The 1st field of type C{int} contains the largest number of concurrently active spritegroup ids.
-The 2nd field of type L{Position} contains a positional reference to the last spritegroup of the concurrently active ones.
+The 2nd field of type L{Position} contains a positional reference
+  to the last spritegroup of the concurrently active ones.
 """
 spritegroup_stats = (0, None)
 
@@ -37,14 +37,24 @@ The 2nd field of type L{Position} contains a positional reference to the spriteg
 """
 a2register_stats = (0, None)
 
+
 def print_stats():
     """
     Print statistics about used ids.
     """
     if spritegroup_stats[0] > 0:
-        generic.print_info("Concurrent spritegroups: {}/{} ({})".format(spritegroup_stats[0], total_action2_ids, str(spritegroup_stats[1])))
+        generic.print_info(
+            "Concurrent spritegroups: {}/{} ({})".format(
+                spritegroup_stats[0], total_action2_ids, str(spritegroup_stats[1])
+            )
+        )
     if a2register_stats[0] > 0:
-        generic.print_info("Concurrent Action2 registers: {}/{} ({})".format(a2register_stats[0], total_tmp_locations, str(a2register_stats[1])))
+        generic.print_info(
+            "Concurrent Action2 registers: {}/{} ({})".format(
+                a2register_stats[0], total_tmp_locations, str(a2register_stats[1])
+            )
+        )
+
 
 class Action2(base_action.BaseAction):
     """
@@ -72,6 +82,7 @@ class Action2(base_action.BaseAction):
                          to be used in this varaction2.
     @type tmp_locations: C{list} of C{int}
     """
+
     def __init__(self, feature, name, pos):
         self.feature = feature
         self.name = name
@@ -79,10 +90,10 @@ class Action2(base_action.BaseAction):
         self.num_refs = 0
         self.id = None
         self.references = []
-        #0x00 - 0x7F: available to user
-        #0x80 - 0xFE: used by NML
-        #0xFF: Used for some house variables
-        #0x100 - 0x10F: Special meaning (used for some CB results)
+        # 0x00 - 0x7F: available to user
+        # 0x80 - 0xFE: used by NML
+        # 0xFF: Used for some house variables
+        # 0x100 - 0x10F: Special meaning (used for some CB results)
         self.tmp_locations = list(range(0x80, 0x80 + total_tmp_locations))
 
     def prepare_output(self, sprite_num):
@@ -99,11 +110,18 @@ class Action2(base_action.BaseAction):
                 if num_used > spritegroup_stats[0]:
                     spritegroup_stats = (num_used, self.pos)
         except IndexError:
-            raise generic.ScriptError("Unable to allocate ID for [random]switch, sprite set/layout/group or produce-block. Try reducing the number of such blocks.", self.pos)
+            raise generic.ScriptError(
+                "Unable to allocate ID for [random]switch, sprite set/layout/group or produce-block."
+                " Try reducing the number of such blocks.",
+                self.pos,
+            )
 
-    def write_sprite_start(self, file, size):
+    def write_sprite_start(self, file, size, extra_comment=None):
         assert self.num_refs == 0, "Action2 reference counting has {:d} dangling references.".format(self.num_refs)
         file.comment("Name: " + self.name)
+        if extra_comment:
+            for c in extra_comment:
+                file.comment(c)
         file.start_sprite(size + 3)
         file.print_bytex(2)
         file.print_bytex(self.feature)
@@ -145,6 +163,7 @@ class Action2(base_action.BaseAction):
             if force_recursive or act2_ref.is_proc:
                 act2_ref.action2.remove_tmp_location(location, True)
 
+
 class Action2Reference:
     """
     Container class to store information about an action2 reference
@@ -155,11 +174,13 @@ class Action2Reference:
     @ivar is_proc: Whether this reference is made because of a procedure call
     @type is_proc: C{bool}
     """
+
     def __init__(self, action2, is_proc):
         self.action2 = action2
         self.is_proc = is_proc
 
-def add_ref(ref, source_action, reference_as_proc = False):
+
+def add_ref(ref, source_action, reference_as_proc=False):
     """
     Add a reference to a certain action2.
     This is needed so we can correctly reserve / free action2 IDs later on.
@@ -181,6 +202,7 @@ def add_ref(ref, source_action, reference_as_proc = False):
     source_action.references.append(Action2Reference(act2, reference_as_proc))
     act2.num_refs += 1
 
+
 def free_references(source_action):
     """
     Free all references to other action2s from a certain action 2/3
@@ -191,7 +213,9 @@ def free_references(source_action):
     for act2_ref in source_action.references:
         act2 = act2_ref.action2
         act2.num_refs -= 1
-        if act2.num_refs == 0: free_action2_ids.append(act2.id)
+        if act2.num_refs == 0:
+            free_action2_ids.append(act2.id)
+
 
 # Features using sprite groups directly: vehicles, canals, cargos, railtypes, airports, roadtypes, tramtypes
 features_sprite_group = [0x00, 0x01, 0x02, 0x03, 0x05, 0x0B, 0x0D, 0x10, 0x12, 0x13]
@@ -200,7 +224,8 @@ features_sprite_layout = [0x04, 0x07, 0x09, 0x0F, 0x11]
 # All features that need sprite sets
 features_sprite_set = features_sprite_group + features_sprite_layout
 
-def make_sprite_group_class(cls_is_spriteset, cls_is_referenced, cls_has_explicit_feature, cls_is_relocatable = False):
+
+def make_sprite_group_class(cls_is_spriteset, cls_is_referenced, cls_has_explicit_feature, cls_is_relocatable=False):
     """
     Metaclass factory which makes base classes for all nodes 'Action 2 graph'
     This graph is made up of all blocks that are eventually compiled to Action2,
@@ -226,7 +251,7 @@ def make_sprite_group_class(cls_is_spriteset, cls_is_referenced, cls_has_explici
     @rtype: C{type}
     """
 
-    #without either references or an explicit feature, we have nothing to base our feature on
+    # without either references or an explicit feature, we have nothing to base our feature on
     assert cls_is_referenced or cls_has_explicit_feature
 
     class ASTSpriteGroup(base_statement.BaseStatement):
@@ -265,15 +290,21 @@ def make_sprite_group_class(cls_is_spriteset, cls_is_referenced, cls_has_explici
         @ivar used_sprite_sets: List of sprite sets used by this node
         @type used_sprite_sets: C{list} of L{SpriteSet}
         """
+
         def __init__(self):
             """
             Subclasses should implement their own __init__ method.
             This method should not be called, because calling a method on a meta class can be troublesome.
             Instead, call initialize(..).
             """
-            raise NotImplementedError('__init__ must be implemented in ASTSpriteGroup-subclass {!r}, initialize(..) should be called instead'.format(type(self)))
+            raise NotImplementedError(
+                (
+                    "__init__ must be implemented in ASTSpriteGroup-subclass {!r},"
+                    " initialize(..) should be called instead"
+                ).format(type(self))
+            )
 
-        def initialize(self, name = None, feature = None, num_params = 0):
+        def initialize(self, name=None, feature=None, num_params=0):
             """
             Initialize this instance.
             This function is generally, but not necessarily, called from the child class' constructor.
@@ -294,10 +325,11 @@ def make_sprite_group_class(cls_is_spriteset, cls_is_referenced, cls_has_explici
             self._referenced_nodes = set()
             self._prepared = False
             self._action2 = {}
-            self.feature_set = set([feature]) if feature is not None else set()
+            self.feature_set = {feature} if feature is not None else set()
             self.name = name
             self.num_params = num_params
             self.used_sprite_sets = []
+            self.optimised = None
 
         def register_names(self):
             if cls_is_relocatable and cls_is_referenced:
@@ -322,7 +354,8 @@ def make_sprite_group_class(cls_is_spriteset, cls_is_referenced, cls_has_explici
             @return: True iff parsing of this node is needed
             @rtype: C{bool}
             """
-            if not cls_is_referenced: return True
+            if not cls_is_referenced:
+                return True
             if not self._prepared:
                 self._prepared = True
                 # copy, since we're going to modify
@@ -339,9 +372,11 @@ def make_sprite_group_class(cls_is_spriteset, cls_is_referenced, cls_has_explici
                     for n in self._referencing_nodes:
                         if n.feature_set != self.feature_set:
                             msg = "Cannot refer to block '{}' with feature '{}', expected feature is '{}'"
-                            msg = msg.format(self.name.value,
-                                    general.feature_name(next(iter(self.feature_set))),
-                                    general.feature_name(n.feature_set.difference(self.feature_set).pop()))
+                            msg = msg.format(
+                                self.name.value,
+                                general.feature_name(next(iter(self.feature_set))),
+                                general.feature_name(n.feature_set.difference(self.feature_set).pop()),
+                            )
                             raise generic.ScriptError(msg, n.pos)
 
                 elif len(self._referencing_nodes) != 0:
@@ -349,10 +384,14 @@ def make_sprite_group_class(cls_is_spriteset, cls_is_referenced, cls_has_explici
                         # Add the features from all calling blocks to the set
                         self.feature_set.update(n.feature_set)
 
-                if len(self._referencing_nodes) == 0:
+                if len(self._referencing_nodes) == 0 and (not self.optimised or self.optimised is self):
                     # if we can be 'not used', there ought to be a way to refer to this block
                     assert self.name is not None
-                    generic.print_warning("Block '{}' is not referenced, ignoring.".format(self.name.value), self.pos)
+                    generic.print_warning(
+                        generic.Warning.OPTIMISATION,
+                        "Block '{}' is not referenced, ignoring.".format(self.name.value),
+                        self.pos,
+                    )
 
             return len(self._referencing_nodes) != 0
 
@@ -377,6 +416,15 @@ def make_sprite_group_class(cls_is_spriteset, cls_is_referenced, cls_has_explici
 
             return self._referencing_nodes
 
+        def optimise(self):
+            """
+            Optimise this sprite group.
+
+            @return: True iff this sprite group has been optimised
+            @rtype: C{bool}
+            """
+            return False
+
         def collect_references(self):
             """
             This function should collect all references to other nodes from this instance.
@@ -384,7 +432,9 @@ def make_sprite_group_class(cls_is_spriteset, cls_is_referenced, cls_has_explici
             @return: A collection containing all links to other nodes.
             @rtype: C{iterable} of L{SpriteGroupRef}
             """
-            raise NotImplementedError('collect_references must be implemented in ASTSpriteGroup-subclass {!r}'.format(type(self)))
+            raise NotImplementedError(
+                "collect_references must be implemented in ASTSpriteGroup-subclass {!r}".format(type(self))
+            )
 
         def set_action2(self, action2, feature):
             """
@@ -432,7 +482,8 @@ def make_sprite_group_class(cls_is_spriteset, cls_is_referenced, cls_has_explici
             @type target_ref: L{SpriteGroupRef}
             """
 
-            if target_ref.name.value == "CB_FAILED": return
+            if target_ref.name.value == "CB_FAILED":
+                return
 
             target = resolve_spritegroup(target_ref.name)
             if target.is_spriteset():
@@ -440,7 +491,12 @@ def make_sprite_group_class(cls_is_spriteset, cls_is_referenced, cls_has_explici
                 # Referencing a spriteset directly from graphics/[random]switch
                 # Passing parameters is not possible here
                 if len(target_ref.param_list) != 0:
-                    raise generic.ScriptError("Passing parameters to '{}' is only possible from a spritelayout.".format(target_ref.name.value), target_ref.pos)
+                    raise generic.ScriptError(
+                        "Passing parameters to '{}' is only possible from a spritelayout.".format(
+                            target_ref.name.value
+                        ),
+                        target_ref.pos,
+                    )
 
                 self.used_sprite_sets.append(target)
             else:
@@ -464,7 +520,7 @@ def make_sprite_group_class(cls_is_spriteset, cls_is_referenced, cls_has_explici
             self._referenced_nodes.remove(target)
             target._referencing_nodes.remove(self)
 
-        #Make metaclass arguments available outside of the class
+        # Make metaclass arguments available outside of the class
         def is_spriteset(self):
             return cls_is_spriteset
 
@@ -473,9 +529,11 @@ def make_sprite_group_class(cls_is_spriteset, cls_is_referenced, cls_has_explici
 
     return ASTSpriteGroup
 
-#list of all registered sprite sets and sprite groups
+
+# list of all registered sprite sets and sprite groups
 spritegroup_list = {}
 
+
 def register_spritegroup(spritegroup):
     """
     Register a sprite group, so it can be resolved by name later
@@ -487,7 +545,8 @@ def register_spritegroup(spritegroup):
     if name in spritegroup_list:
         raise generic.ScriptError("Block with name '{}' has already been defined".format(name), spritegroup.pos)
     spritegroup_list[name] = spritegroup
-    nml.global_constants.spritegroups[name] = name
+    global_constants.spritegroups[name] = name
+
 
 def resolve_spritegroup(name):
     """
@@ -502,4 +561,3 @@ def resolve_spritegroup(name):
     if name.value not in spritegroup_list:
         raise generic.ScriptError("Unknown identifier encountered: '{}'".format(name.value), name.pos)
     return spritegroup_list[name.value]
-
