@@ -13,7 +13,7 @@ You should have received a copy of the GNU General Public License along
 with NML; if not, write to the Free Software Foundation, Inc.,
 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA."""
 
-from nml import expression, nmlop
+from nml import nmlop
 
 callbacks = 0x14 * [{}]
 
@@ -43,7 +43,7 @@ general_vehicle_cbs = {
 
 # Function to convert vehicle length to the actual property value, which is (8 - length)
 def vehicle_length(value):
-    return expression.BinOp(nmlop.SUB, expression.ConstantNumeric(8, value.pos), value, value.pos)
+    return nmlop.SUB(8, value)
 
 # Trains
 callbacks[0x00] = {
@@ -68,6 +68,7 @@ callbacks[0x00] = {
     'purchase_tractive_effort_coefficient' : {'type': 'cb', 'num': 0x36, 'var10': 0x1F, 'purchase': 2},
     'bitmask_vehicle_info'                 : {'type': 'cb', 'num': 0x36, 'var10': 0x25},
     'cargo_age_period'                     : {'type': 'cb', 'num': 0x36, 'var10': 0x2B},
+    'curve_speed_mod'                      : {'type': 'cb', 'num': 0x36, 'var10': 0x2E},
     'create_effect'                        : {'type': 'cb', 'num': 0x160},
 }
 callbacks[0x00].update(general_vehicle_cbs)
@@ -216,8 +217,8 @@ def cargo_profit_value(value):
     # (amount * price_factor * (cb_result * 329 / 256)) / 256
     # This allows us to report a factor of 256 in the documentation, which makes a lot more sense than 199.804...
     # Not doing the division here would improve accuracy, but limits the range of the return value too much
-    value = expression.BinOp(nmlop.MUL, value, expression.ConstantNumeric(329), value.pos)
-    return expression.BinOp(nmlop.DIV, value, expression.ConstantNumeric(256), value.pos)
+    value = nmlop.MUL(value, 329)
+    return nmlop.DIV(value, 256)
 
 callbacks[0x0B] = {
     'profit'         : {'type': 'cb', 'num':  0x39, 'flag_bit': 0, 'value_function': cargo_profit_value},
